// Deez Clock
/* HUES
    Red (0..)
    Yellow (42..)
    Green (85..)
    Aqua (128..)
    Blue (171..)
    Purple (213..)
*/

#include <Wire.h>
#include <Time.h>
#include <DS1307RTC.h>
#include <FastLED.h>

#define LED_DATA_PIN 7
#define LED_COUNT 144
#define TIME_MARKER CHSV(0,0,100)

// Zones
int zone_hue[4];
int zone_start[4];
int zone_end[4];

// Dimming Times
int dimmer_start;
int dimmer_end;
int dimmer_value;

#define TIME_MARKER_COLOR CRGB::White
#define TIME_MARKER_MAX_BRIGHTNESS 160
#define HOUR_MARKER_COLOR CRGB::Red
#define HOUR_MARKER_BRIGHTNESS 50

#define HOUR_CHIME_COLOR CHSV(0,0,150);

#define HOUR12_MARKER_COLOR CRGB::Blue
#define MOVING_CURSOR_COLOR CRGB::Yellow;
#define MOVING_CURSOR_BRIGHTNESS 50;
#define MOVING_CURSOR_SCALE 15000//Seconds to travel all LEDs

#define CLOCK_POWER_PIN A3// A3= UNO Pins / A1=Leo
#define CLOCK_GND_PIN A2 // A2= UNO Pins / A0=Leo
#define PULSE_PIN 13
#define SHOW_SERIAL

CRGB leds[LED_COUNT+1];

CRGB color[4];
int setting[6];

int prev_sec = 0;
boolean sec_pulse = true;
int led_hour = LED_COUNT/24;

int hours[24];

long cycles_per_sec = 90;
long cycle_count = 90;
long cycles = 0;

int timemarker = 0;
int chime = 0;

void setup() {

  pinMode(PULSE_PIN,OUTPUT);
  pinMode(CLOCK_POWER_PIN,OUTPUT);
  pinMode(CLOCK_GND_PIN,OUTPUT);
  digitalWrite(CLOCK_POWER_PIN,HIGH);
  digitalWrite(CLOCK_GND_PIN,LOW);

  setTime();
  
  zone_hue[0] = 96;
  zone_start[0] = 11*6; // LED#
  zone_end[0] = 19*6; // LED#

  dimmer_start = 18;
  dimmer_end = 11;
  dimmer_value = 150;
  
  color[0] = HOUR_MARKER_COLOR;
  color[1] = HOUR12_MARKER_COLOR;
  color[2] = MOVING_CURSOR_COLOR;
  color[3] = HOUR_CHIME_COLOR;
  
  setting[0] = HOUR_MARKER_BRIGHTNESS;
  setting[1] = MOVING_CURSOR_SCALE;
  setting[2] = MOVING_CURSOR_BRIGHTNESS;
  setting[3] = TIME_MARKER_COLOR;
  setting[4] = TIME_MARKER_MAX_BRIGHTNESS;
  setting[5] = 30; // Pulse Rate
  
  color[2].maximizeBrightness(setting[2]);


  Serial.begin(9600);
  delay(200);
  Serial.println("DS1307RTC Read Test");
  Serial.println("-------------------");

  setSyncProvider(RTC.get);   // the function to get the time from the RTC

// Leds
    FastLED.addLeds<NEOPIXEL, LED_DATA_PIN>(leds, LED_COUNT);
    FastLED.showColor(0); 

//Set Hour LEDs
  
  for(int i=0;i<24;i++) {
      hours[i] = i*led_hour;
  }

//start_fade();
time_chime();

}

void loop() {

//Clear all first
  dimmer(255);
  
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  
  timemarker = ((hour()*60) + (minute()))/(60/led_hour);
  
  secPulse();
  
  displayHours();
  
  displayZones();
  
  display12();
  
  secondsCursor();
  
  timeMarker();
  chime = minute();
  if((chime == 0 || chime == 15 || chime == 30 || chime == 45) && second() == 0) time_chime();
  
  FastLED.show();
}

void secPulse() {
  //cycle_count++;
  if (prev_sec != second()) {
    prev_sec = second();
    sec_pulse = !sec_pulse;
    digitalWrite(PULSE_PIN, (sec_pulse == true?HIGH:LOW));  
  }
}

void displayHours() {

  // Lets start with hour positions
  for(int i=0; i < 24;i++) {
      fadeMarker(hours[i],color[0]);
  }
  
}

void displayZones() {
  static int b = 0;
  // Lets start with A1 positions
  for(int i=0; i < LED_COUNT;i++) {
    if (i >= zone_start[0] && i<zone_end[0]) {
      b = max(leds[i].r,max(leds[i].g,leds[i].b));
      leds[i] = CHSV(zone_hue[0],255,255);
      leds[i].maximizeBrightness(b);
    }
  }
}

void display12() {
        fadeMarker(12*led_hour,color[1]);
}

void secondsCursor() {
  static int cursor_led;
  static int cursor_led_prev;
  static int cycles;
  static int prev_sec;
  static CRGB temp_color;
  
  if (cycles < 255) {
    cycles += 5;
  }

  if(cursor_led >= LED_COUNT) cursor_led = 0;

  if(prev_sec != second()) {
    cursor_led++;
    cursor_led_prev = cursor_led-1;
    prev_sec = second();
    cycles = 0;

      if (true) {
        Serial.print(hour());
        Serial.print(':');
        Serial.print(minute());
        Serial.print(':');
        Serial.println(second());
      }
    }

//Lets fade between 2 pixels for smooth flow

//  Serial.println((int)percent);
  temp_color = color[2];
  temp_color.nscale8(map(cycles,0,255,255,0));
  leds[cursor_led_prev] += temp_color;

  temp_color = color[2];
  temp_color.nscale8(cycles);
  leds[cursor_led] += temp_color;

}

void timeMarker() {
static int a = 0;
static boolean d = true;
static unsigned long t = 0;

  // Time Marker
  leds[timemarker] = setting[3];

  if(millis() > t) { a++; t = millis() + setting[5]; }

  if (a >= setting[4]) {d = !d;a=0;}
  
  if (d) {
    leds[timemarker].nscale8(map(a,0,setting[4],setting[4],0));
  } else {
    leds[timemarker].nscale8(a);
  }

}

void dimmer(uint8_t d) {
    for(int i = 0; i < LED_COUNT; i++) {
        leds[i].fadeToBlackBy(d);
    }
}

// ============= General Stuff

void fadeMarker(int led, CRGB c) {
    for (int f=0;f<led_hour;f++) {
      leds[led+f] = c;
      leds[led+f].nscale8(f==0?setting[0]:(setting[0]/4)-(((setting[0]/4)/(led_hour-1))*f));
    }
}

void time_chime() {
  static CRGB wheel;
  static uint8_t c;
  
        displayHours();
        displayZones();
        display12();
      for(int i = 0; i < LED_COUNT; i++) {
        c += 3;
        wheel.setHue(c);
        leds[i] += wheel; //color[3];
        for(int a = 0; a < i; a++) {
          leds[a].fadeToBlackBy(100);
        }
        FastLED.show();
        delay(40);
      }
      
      while(leds[LED_COUNT-1]) {
        for(int a = 0;a<LED_COUNT; a++) {
          leds[a].fadeToBlackBy(100);
        }
          FastLED.show();
          delay(40);
      }
      start_fade();
}

void start_fade() {
// Make all white
      for(int i = 0; i < LED_COUNT; i++) {
        displayHours();
        displayZones();
        display12();
        for(int a = i; a < LED_COUNT; a++) {
          leds[a].fadeToBlackBy(255);
        }
        FastLED.show();
        delay(20);
      }
}

